#ifndef HAS_CORE
#define HAS_CORE 0
#endif

// If we have a discontinuous texture (e.g. a swizzled, tiled texture) we need to use the gradient fetch functions so the LOD doens't
// go insane: http://hacksoflife.blogspot.com/2010/02/how-to-change-your-uv-map-on-fly.html
// So...if we have the extension, grab it.  Otherwise use a macro to dumb down our shader fetches to what all GLSL supports.


#if HAS_CORE
	#define texture2D		texture
	#define texture2DGradARB(a,b,c,d)	textureGrad(a,b,c,d)
	#define shadow2DGradARB(a,b,c,d)	shadowGrad(a,b,c,d)
#elif __VERSION__ >= 130
	#define texture2DGradARB(a,b,c,d)	textureGrad(a,b,c,d)
	#define shadow2DGradARB(a,b,c,d)	vec4(textureGrad(a,b,c,d))
	#define shadow2DArrayGradARB(a,b,c,d)	vec4(textureGrad(a,b,c,d))

	#define shadow2DArrayARB(a,b) vec4(texture(a,b))
#elif defined(GL_EXT_gpu_shader4)
	#extension GL_EXT_gpu_shader4 : enable
	#define texture2DGradARB(a,b,c,d)	texture2DGrad(a,b,c,d)
	#define shadow2DGradARB(a,b,c,d)	shadow2DGrad(a,b,c,d)
	#define shadow2DArrayGradARB(a,b,c,d)	shadow2DArrayGrad(a,b,c,d)
	#define shadow2DArrayARB(a,b) shadow2DArray(a,b)
#elif HAS_SHADER_LOD
	// Ben says: Intel HD driver defines GL_ARB_shader_texture_lod as 1 on GL4.0 driver even though
	// the extension is not available; use of shadow2DGradARB then fails compile.  So switch to a 
	// symbol defined by the sim from the extension string, which is accurate.  Intel can export
	// GPU-shader-4 to expose gradient sampling.
	#extension GL_ARB_shader_texture_lod : enable
#else
	#define texture2DGradARB(a,b,c,d) texture2D(a,b)
	#define shadow2DArrayGradARB(a,b,c,d) shadow2DArray(a,b)
	#define shadow2DArrayARB(a,b) shadow2DArray(a,b)
#endif

#if defined(GL_ARB_gpu_shader5)
	#extension GL_ARB_gpu_shader5 : enable
#endif

#if HAS_UBO
	#extension GL_ARB_uniform_buffer_object : enable
#endif



mat3 mat3_cast(mat4 m)
{ 
	return mat3(m[0].xyz,m[1].xyz,m[2].xyz);
}

#define mat3(X) mat3_cast(X)
